import amqp from "amqplib/callback_api";

interface IConfig {
  rabbitMQUrl: string;
}

const workerId: number = 1;
const personalNumber: number = 12345;

const worker = {
  id: workerId,
  personalNumber,
  firstName: "John",
  lastName: "Doe",
  userName: "super-john-doe",
  role: "SIMPLE_WORKER",
  rating: 5,
  sectionId: 1,
  createdAt: new Date().toISOString(),
  updatedAt: new Date().toISOString()
};

const taskList = [...Array(100).keys()].map((index: number) => ({
  id: index + 1,
  name: `Task ${index + 1}`,
  status: 1,
  sectionId: 1,
  rating: index % 5,
  taskTypeId: 1,
  workerId,
  createdAt: new Date().toISOString(),
  updatedAt: new Date().toISOString()
}));

const mockDataList = [
  { type: "worker", payload: { ...worker } },
  ...taskList.map(task => ({
    type: "task",
    payload: { ...task }
  }))
];

export default async ({ task, taskService }) => {
  try {
    const variables: IConfig = task.variables.getAll();

    await amqp.connect(variables.rabbitMQUrl, (error, connection) => {
      if (error) throw error;

      connection.createChannel(async (error, channel) => {
        if (error) throw error;

        channel.assertQueue(personalNumber.toString(), { durable: false });

        for (const mock of mockDataList) {
          await channel.sendToQueue(
            personalNumber.toString(),
            Buffer.from(JSON.stringify(mock))
          );
        }

        await taskService.complete(task);
      });
    });
  } catch (error) {
    console.log("error", error);
  }
};
