import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import format from 'string-template';
import _ from 'lodash';

import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  entity: string;
  id: string;
}

interface IResult {
  payload: string;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  let retries = task.retries;

  if (_.isNull(retries)) {
    retries = 10;
  }

  try {
    const backendUrlTemplate: string = variables.BACKEND_URL + '/{entity}/{id}';
    const entity = await axios({
      method: 'delete',
      url: format(backendUrlTemplate, {
        entity: variables.entity,
        id: variables.id
      }),
      headers: { 'X-Username': 'microservice [DEL]' }
    }).then(response => response.data);

    const result: IResult = {
      payload: JSON.stringify(entity)
    };

    await taskService.complete(task, new Variables().setAll(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message,
      retries: retries - 1,
      retryTimeout: 60000,
    });
  }
};
