import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import format from 'string-template';
import _ from 'lodash';

import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  entity: string;
  id: string;
  payload: string;
}

interface IResult {
  payload: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  let retries = task.retries;

  if (_.isNull(retries)) {
    retries = 10;
  }

  try {
    const backendUrlTemplate: string = variables.BACKEND_URL + '/{entity}/{id}';
    const entity = await axios({
      method: 'patch',
      url: format(backendUrlTemplate, {
        entity: variables.entity,
        id: variables.id,
      }),
      headers: {
        'Content-Type': 'application/merge-patch+json',
        'X-Username': 'microservice [PATCH]',
      },
      data: JSON.parse(variables.payload)
    }).then(response => response.data);

    const result: IResult = {
      payload: {
        value: JSON.stringify(entity),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message,
      retries: retries - 1,
      retryTimeout: 60000,
    });
  }
};
