import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import format from 'string-template';

import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  entity: string;
  payload: string;
}

interface IResult {
  payload: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const backendUrlTemplate: string = variables.BACKEND_URL + '/{entity}';
    const entity = await axios({
      method: 'post',
      url: format(backendUrlTemplate, {
        entity: variables.entity
      }),
      headers: {
        Accept: 'application/ld+json',
        'Content-Type': 'application/ld+json',
        'X-Username': 'microservice [SET]',
      },
      data: variables.payload
    }).then(response => response.data);

    const result: IResult = {
      payload: {
        value: JSON.stringify(entity),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
