import { Variables } from "camunda-external-task-client-js";

import _ from "lodash";

import logging from "../logging";

interface IData {
  sourceObject: string;
  targetObject: string;
}

interface IResult {
  payload: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const sourceObject = JSON.parse(variables.sourceObject);
    const targetObject = JSON.parse(variables.targetObject);

    const reducedTarget = _.reduce(
      sourceObject,
      function(result, value, key) {
        if (_.has(targetObject, key) || _.isNull(value)) {
          result[key] = value;
        }

        return result;
      },
      {}
    );

    const result: IResult = {
      payload: {
        value: _.isEqual(sourceObject, reducedTarget),
        type: "Boolean"
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
