import _ from 'lodash';

import logging from '../logging';
import debug from '../debug';

interface IConfig {
  ELK_URL: string; // not needed? Should be set in ENV already
}

interface IData {
  type: string;
  message: string;
  payload: string;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const message = {
      type: variables.type,
      payload: JSON.parse(variables.payload)
    };

    debug({
      type: variables.type,
      message: variables.message,
      payload: message,
      variables: variables,
      filename: __filename,
    });

    await taskService.complete(task);
  } catch (error) {
    await logging({ error, variables, filename: __filename, type: 'RABBITMQ' });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  } finally {
    // Do something nasty
  }
};
