import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import _ from 'lodash';

import constants  from '../const/constants';
import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
  propertyEntity: string;
}

interface IData {
  rawProperties: string;
}

interface IResult {
  patch: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    let rawProperties = JSON.parse(variables.rawProperties);

    const properties = [];

    if (!_.isArray(rawProperties)) {
      const tmp = [];

      tmp.push(rawProperties);

      rawProperties = tmp;
    }

    let rowAmount = null

    for (const rawProperty of rawProperties) {
      const propertyId = rawProperty['Параметр_Key'];

      const propertyPromise = _.isEmpty(propertyId)
        ? new Promise(resolve => resolve(null))
        : axios({
            method: 'get',
            url:
              variables.BACKEND_URL +
              '/' +
              variables.propertyEntity +
              '/' +
              propertyId,
            headers: {
              Accept: 'application/ld+json',
              'Content-Type': 'application/merge-patch+json',
              'X-Username': 'microservice [G PROPS]',
            }
          })
            .then(response => response.data)
            .catch(error => {
              if (error.response.status === 404) {
                return null;
              }

              throw error;
            });

      const parentProperty = await propertyPromise;

      if (_.isNull(parentProperty)) {
        continue;
      }

      if (
        _.isNull(rowAmount)
        && (
          // rawProperty["Параметр_Key"] === 'c59f47ec-2607-11e9-a2e3-005056a3e110'
          rawProperty['Параметр_Key'] === constants.rowId
        )
      ) {
        rowAmount = 0;
      }

      const property = {
        id: rawProperty['Параметр_Key'],
        name: parentProperty['name'],
        description: parentProperty['description'],
        regexp: !!rawProperty['РегулярноеВыражение']
          ? '' + rawProperty['РегулярноеВыражение']
          : '' + parentProperty['regexp'],
        type: parentProperty['type'],
        value: '' + rawProperty['РезультатВыполнения'],
        required: rawProperty['Обязательный'],
        stage: rawProperty['ПриЗавершении'] ? 'finish' : 'start'
      };

      properties.push(property);

      if (rawProperty['Параметр_Key'] === constants.trolleyId && !rawProperty['ПриЗавершении'] && false) {
        const endTrolleyProperty = {
          id: rawProperty['Параметр_Key'],
          name: parentProperty['name'],
          description: parentProperty['description'],
          regexp: !!rawProperty['РегулярноеВыражение']
            ? '' + rawProperty['РегулярноеВыражение']
            : '' + parentProperty['regexp'],
          type: parentProperty['type'],
          value: '' + rawProperty['РезультатВыполнения'],
          required: rawProperty['Обязательный'],
          stage: !rawProperty['ПриЗавершении'] ? 'finish' : 'start'
        };

        properties.push(endTrolleyProperty);
      }
    }

    const result: IResult = {
      patch: {
        value: JSON.stringify({
          properties: properties,
          rowAmount: rowAmount,
        }),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          // objectTypeName: 'java.util.ArrayList<java.util.LinkedHashMap<java.lang.Object,java.lang.Object>>',
          serializationDataFormat: 'application/json'
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
