import { Variables } from "camunda-external-task-client-js";

import * as R from "ramda";

import logging from "../logging";

interface IData {
  collection: string;
  loopCounter: number;
}

interface IResult {
  payload: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const result: IResult = {
      payload: R.compose(
        R.nth(variables.loopCounter),
        JSON.parse
      )(variables.collection)
    };

    await taskService.complete(task, new Variables().setAll(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
