import { Variables } from 'camunda-external-task-client-js';

import uuidv4 from 'uuid/v4';

import logging from '../logging';

interface IData {
  meta: string;
  payload: string;
  type: string;
}

interface IResult {
  payload: string;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const result: IResult = {
      payload: JSON.stringify({
        id: uuidv4(),
        raw: JSON.parse(variables.payload),
        type: variables.type,
        meta: JSON.parse(variables.meta),
        isProcessed: false,
        triesAmount: 0
      })
    };

    await taskService.complete(task, new Variables().setAll(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
