import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import format from 'string-template';
import _ from 'lodash';

import logging from '../logging';

interface IConfig {
  ONEC_URL: string;
  ONEC_UNM: string;
  ONEC_PWD: string;
  entity: string;
}

interface IResult {
  payload?: object;
  isAxiosError: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig = task.variables.getAll();

  try {
    const urlTemplate: string = variables.ONEC_URL + '/hs/NewMUP/{entity}';

    const entity = await axios({
      method: 'get',
      auth: {
        username: variables.ONEC_UNM,
        password: variables.ONEC_PWD
      },
      url: encodeURI(
        format(urlTemplate, {
          entity: variables.entity
        })
      )
    }).then(response => response.data);

    if (!entity) throw new Error('Entity not found');

    let result: IResult;

    if (_.isObject(entity)) {
      result = {
        payload: {
          value: JSON.stringify(entity),
          type: 'Object',
          valueInfo: {
            objectTypeName:
              'java.util.ArrayList<java.util.LinkedHashMap<java.lang.Object,java.lang.Object>>',
            serializationDataFormat: 'application/json'
          }
        },
        isAxiosError: {
          value: false,
          type: 'Boolean',
        },
      };
    } else {
      result = {
        payload: {
          value: entity,
          type: 'String'
        },
        isAxiosError: {
          value: false,
          type: 'Boolean',
        },
      };
    }

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    let result: IResult;
    result = {
      isAxiosError: {
        value: true,
        type: 'Boolean',
      },
    }
    await taskService.complete(task, new Variables().setAllTyped(result));
  }
};
