import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import format from 'string-template';
import _ from 'lodash';

import logging from '../logging';

interface IConfig {
  ONEC_URL: string;
  ONEC_UNM: string;
  ONEC_PWD: string;
}

interface IData {
  taskId: string;
}

interface IResult {
  payload?: object;
  error: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {

    const entity = 'Document_актЗаказНарядНаРаботы';

    const urlTemplate: string =
      variables.ONEC_URL + "/odata/standard.odata/{entity}(guid:'{taskId}')?$format=json";

    const auth = {
      username: variables.ONEC_UNM,
      password: variables.ONEC_PWD,
    }

    const payload = await axios({
      method: 'get',
      auth: !_.isEmpty(variables.ONEC_UNM) ? auth : null,
      url: encodeURI(
        format(urlTemplate, {
          entity: entity,
          taskId: variables.taskId
        })
      )
    })
      .then(response => response.data)
      .catch(async error => {
        if (error.response.status === 404) {
          return null
        }

        await logging({ error, variables, filename: __filename });

        return null
      });

    const result: IResult = {
      payload: {
        value: JSON.stringify(payload),
        type: 'Object',
        valueInfo: {
          objectTypeName: 'java.util.HashMap<java.lang.Object, java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      error: {
        value: payload ? false : true,
        type: 'Boolean'
      },
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
