/* tslint:disable no-null-keyword */

import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import _ from 'lodash';

import logging from '../logging';
import moment from 'moment';

interface IConfig {
  ONEC_URL: string;
  ONEC_UNM: string;
  ONEC_PWD: string;
}

interface IData {
  taskPost: string;
  taskPatch: string;
}

interface IResult {
  isProcessed: object;
  taskNumber?: object;
  syncedAt: object;
  rawResponse: object;
  status: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const taskData = JSON.parse(variables.taskPost);

    let taskNumber = taskData.Number;
    let status = null;

    const oneCUrlGetPatch: string =
      variables.ONEC_URL +
      '/odata/standard.odata/'
        + "Document_актЗаказНарядНаРаботы(guid:'" + taskData.Ref_Key + "')?$format=json";

    const oneCUrlPost: string =
      variables.ONEC_URL +
      '/odata/standard.odata/'
        + 'Document_актЗаказНарядНаРаботы?$format=json';

    const oneCUrlInvoice: string =
      variables.ONEC_URL +
      '/odata/standard.odata/'
        + "Document_актЗаказНарядНаРаботы(guid:'" + taskData.Ref_Key + "')/Post()";

    const auth = {
      username: variables.ONEC_UNM,
      password: variables.ONEC_PWD,
    };

    const rawResponse = await axios({
      method: 'get',
      // tslint:disable-next-line: no-null-keyword
      auth: !_.isEmpty(variables.ONEC_UNM) ? auth : null,
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json'
      },
      url: encodeURI(oneCUrlGetPatch),
    })
      .then(async () => {
        return await axios({
          method: 'patch',
          // tslint:disable-next-line: no-null-keyword
          auth: !_.isEmpty(variables.ONEC_UNM) ? auth : null,
          headers: {
            Accept: 'application/json',
            'Content-Type': 'application/json'
          },
          url: encodeURI(oneCUrlGetPatch),
          data: variables.taskPatch
        }).then(async response => {
          taskNumber = response.data.Number;
          status = response.status;

          if (response.data.DeletionMark != true && response.data['Факт']) {
            await axios({
              method: 'post',
              // tslint:disable-next-line: no-null-keyword
              auth: !_.isEmpty(variables.ONEC_UNM) ? auth : null,
              url: encodeURI(oneCUrlInvoice),
            }).then(response => response.status);
          }

          return response.data;
        });
      }) // 200 OK Task Exists, do nothing
      .catch(async error => {
        // Invalid request data without response
        // if (error.isAxiosError) {
        //   throw error;
        // }
        if (_.get(error, 'response.status', 500) === 404) { // 404 not found, need to be created
          return await axios({
            method: 'post',
            // tslint:disable-next-line: no-null-keyword
            auth: !_.isEmpty(variables.ONEC_UNM) ? auth : null,
            headers: {
              Accept: 'application/json',
              'Content-Type': 'application/json'
            },
            url: encodeURI(oneCUrlPost),
            data: variables.taskPost
          }).then(async response => {
            if (response.status === 201) {
              taskNumber = response.data.Number;
              status = response.status;

              if (response.data['Факт']) {
                await axios({
                  method: 'post',
                  // tslint:disable-next-line: no-null-keyword
                  auth: !_.isEmpty(variables.ONEC_UNM) ? auth : null,
                  url: encodeURI(oneCUrlInvoice),
                }).then(response => response.data);
              }

              return response.data;
            }
          });
        }

        // throw error;
      });

    const result: IResult = {
      isProcessed: {
        value: true,
        type: 'Boolean'
      },
      taskNumber: {
        value: taskNumber ? taskNumber : '',
        type: 'String'
      },
      syncedAt: {
        value: moment().format(),
        type: 'String'
      },
      rawResponse: {
        value: JSON.stringify(rawResponse),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      status: {
        value: status,
        type: 'String'
      },
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    const result: IResult = {
      isProcessed: {
        value: false,
        type: 'Boolean'
      },
      taskNumber: {
        value: '',
        type: 'String'
      },
      syncedAt: {
        value: false,
        type: 'Boolean'
      },
      rawResponse: {
        value: JSON.stringify({}),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      status: {
        value: _.get(error, 'message', '500 Server Error'),
        type: 'String'
      },
    };
    await taskService.complete(task, new Variables().setAllTyped(result));
  }
};
