/* tslint:disable no-null-keyword */

import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';
import _ from 'lodash';

import logging from '../logging';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  history: string;
}

interface IResult {
  payload: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const history = JSON.parse(variables.history);

    if (history.active) {
      const workerId = history.workerId;

      const worker = {
        positionId: null,
        departmentId: null,
        rawPositionId: history.positionId,
        rawDepartmentId: _.isEmpty(history.departmentId)
          ? null
          : await axios({
              method: 'get',
              url:
                variables.BACKEND_URL + '/departments/' + history.departmentId,
              headers: {
                Accept: 'application/ld+json',
                'Content-Type': 'application/merge-patch+json',
                'X-Username': 'microservice [HIST PROCESS 1]',
              }
            })
              .then(response => response.data.altId || response.data.id)
              .catch(error => {
                if (error.response.status === 404) {
                  return null;
                }

                throw error;
              }),
        isActive: history.event === 'Увольнение' ? false : true
      };

      if (!!workerId) {
        await axios({
          method: 'patch',
          url: variables.BACKEND_URL + '/workers/' + workerId,
          headers: {
            Accept: 'application/ld+json',
            'Content-Type': 'application/merge-patch+json',
            'X-Username': 'microservice [HIST PROCESS 2]',
          },
          data: worker
        })
          .then(() => true)
          .catch(error => {
            if (error.response.status === 404) {
              return false;
            }

            throw error;
          });
      }
    }

    const result: IResult = {
      payload: {
        value: JSON.stringify(history),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          // objectTypeName: 'java.util.ArrayList<java.util.LinkedHashMap<java.lang.Object,java.lang.Object>>',
          serializationDataFormat: 'application/json'
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
