/* tslint:disable no-null-keyword */

import _ from 'lodash';
import logging from '../logging';
import axios from 'axios';
import uuidv4 from 'uuid/v4';
import moment from 'moment';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  properties: string;
  workerId: string;
  taskId: string;
  plantId: string;
  barcode: string;
}

export default async ({ task, taskService }) => {
  const variables: IData & IConfig = task.variables.getAll();

  try {
    const properties = JSON.parse(variables.properties);

    const backendUrl: string = variables.BACKEND_URL + '/metrics';
    const taskUrl: string = variables.BACKEND_URL + '/tasks';
    const codeUrl: string = variables.BACKEND_URL + '/code_metrics';

    for (const p of properties) {
      if (_.isEmpty(p.value)) {
        continue;
      }

      if (variables.taskId) {
        const greenhouseId = await axios({
          method: 'get',
          url: `${taskUrl}/${variables.taskId}`,
          headers: {
            Accept: 'application/json',
            'Content-Type': 'application/json',
            'X-Username': 'microservice [PROPS CREATE GET TASK]',
          },
          params: {
            groups: [
              'mobile',
            ],
            properties: [
              'id',
              'greenhouseId'
            ],
          },
        }).then(response => response.data)
          .then(task => task.greenhouseId);

        const plantCode = {
          // id: variables.barcode,
          code: variables.barcode,
          collectableId: p.parentId && p.parentId || null,
          plantId: variables.plantId && variables.plantId || null,
          greenhouseId,
        };

        const plantCodeEx = await axios({
          method: 'get',
          url: codeUrl,
          headers: {
            Accept: 'application/json',
            'Content-Type': 'application/json',
            'X-Username': 'microservice [PROPS CREATE GET CODE METRIC]',
          },
          params: {
            code: plantCode.code,
            collectableId: plantCode.collectableId,
            page: 1,
            itemsPerPage: 1,
          },
        }).then(response => response.data)
          .then(codes => codes[0] || null);

        if (plantCodeEx) {
          await axios({
            method: 'patch',
            url: `${codeUrl}/${plantCodeEx.id}`,
            data: JSON.stringify(plantCode),
            headers: {
              Accept: 'application/ld+json',
              'Content-Type': 'application/merge-patch+json',
              'X-Username': 'microservice [PROPS CREATE CODE METRIC PATCH]',
            }
          })
            .then(response => console.info('SUCCESS: Code Metric patched!', response.data.id))
            .catch(error => console.error('ERROR: Code Metric PATCH fail!', error));
        } else {
          await axios({
            method: 'post',
            url: codeUrl,
            headers: {
              Accept: 'application/ld+json',
              'Content-Type': 'application/ld+json',
              'X-Username': 'microservice [PROPS CREATE CODE METRIC POST]',
            },
            data: JSON.stringify(plantCode)
          }).then(response => console.info('SUCCESS: Code Metric created!', response.data.id))
            .catch(error => console.error('ERROR: Code Metric POST fail!', error));
        }
      }

      await axios({
        method: 'post',
        url: backendUrl,
        headers: {
          Accept: 'application/ld+json',
          'Content-Type': 'application/ld+json',
          'X-Username': 'microservice [PROPS CREATE]',
        },
        data: JSON.stringify({
          id: uuidv4(),
          code: variables.barcode,
          property: p.id ? 'collectables/' + p.id : null,
          parent: p.parentId ? 'collectables/' + p.parentId : null, // replace to actual group id
          task: variables.taskId ? 'tasks/' + variables.taskId : null,
          worker: variables.workerId ? 'workers/' + variables.workerId : null,
          plant: variables.plantId ? 'plant_phenologies/' + variables.plantId : null,
          value: p.value,
          registeredAt: p.timestamp,
          raw: p,
        })
      }).then(response => console.info('SUCCESS: Property created!', response.data.id));
      // .catch(err => {
      //   console.error(err);
      // });
    }

    await taskService.complete(task);
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};