import amqp from 'amqplib';

import logging from '../logging';

interface IConfig {
  RABBITMQ_URL: string;
}

interface IData {
  queue: string;
  type: string;
  payload: string;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const connection = await amqp.connect(variables.RABBITMQ_URL);

    const channel = await connection.createChannel();

    channel.assertQueue(variables.queue, {
      durable: false
    });

    const message = {
      type: variables.type,
      payload: JSON.parse(variables.payload)
    };

    await channel.sendToQueue(
      variables.queue,
      Buffer.from(JSON.stringify(message))
    );

    await taskService.complete(task);
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
