import { Variables } from "camunda-external-task-client-js";

import * as R from "ramda";

import logging from "../logging";

interface IData {
  rangeStart: number;
  rangeEnd: number;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const result = {
      collection: {
        value: JSON.stringify(
          R.range(variables.rangeStart, variables.rangeEnd)
        ),
        type: "Object",
        valueInfo: {
          objectTypeName: "java.util.List<java.lang.Integer>",
          serializationDataFormat: "application/json"
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
