import { Variables } from "camunda-external-task-client-js";
import getUuid from "uuid-by-string";

import logging from "../logging";


interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw ;

    const uuidHash = getUuid(raw['Ref_Key'] + raw["Должность_Key"]);

    const applicationData = {
      id: uuidHash, //record['Recorder'],
      raw: raw,
      rawPositionId: raw['Должность_Key'] === '00000000-0000-0000-0000-000000000000' 
        ? null 
        : raw['Должность_Key'],
      rawSectionId: raw['Ref_Key'] === '00000000-0000-0000-0000-000000000000' 
        ? null 
        : raw['Ref_Key'],
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   applicationData, 
    // );    

    const result: IResult = {
      payload: {
        value: JSON.stringify(applicationData),
        type: 'Object',
        valueInfo: {
          objectTypeName: 'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json',
        },
      },
      entity: {
        value: 'applications',
        type: 'String',
      },
      id: {
        value: applicationData.id,
        type: 'String',
      },
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
