import { Variables } from "camunda-external-task-client-js";

import _ from "lodash";

import logging from "../logging";

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    let altId = null;

    const altDep = _.find(raw["ДополнительныеРеквизиты"], {
      //Свойство_Key: "ee26efed-2541-11e9-a2df-005056a3cd19", // || "c4568c94-53dc-11e8-a2c8-005056a3cd19"
      Значение_Type: "StandardODATA.Catalog_СтруктураПредприятия",
    });

    if (!!altDep) {
      altId = !_.isEmpty(altDep["Значение"]) ? altDep["Значение"] : null;
    }

    const department = {
      id: raw.Ref_Key,
      name: raw["Description"],
      rawParentId:
        raw["Parent_Key"] === "00000000-0000-0000-0000-000000000000"
          ? null
          : raw["Parent_Key"],
      rawAltId: altId === "00000000-0000-0000-0000-000000000000" ? null : altId,
      raw: raw
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   department,
    // );

    const result: IResult = {
      payload: {
        value: JSON.stringify(department),
        type: "Object",
        valueInfo: {
          objectTypeName:
            "java.util.HashMap<java.lang.Object,java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      },
      entity: {
        value: "departments",
        type: "String"
      },
      id: {
        value: department.id,
        type: "String"
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
