/* tslint:disable no-null-keyword */

import { Variables } from 'camunda-external-task-client-js';

import logging from '../logging';

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const greenhouse = {
      id: raw.Ref_Key,
      name: raw['Code'],
      description: raw['Description'],
      parentId: null,
      rawParentId: raw['Parent_Key'] === '00000000-0000-0000-0000-000000000000'
        ? null
        : raw['Parent_Key'],
      isFolder: raw['IsFolder'],
      departmentId: null,
      rawDepartmentId: raw['Подразделение_Key'] === '00000000-0000-0000-0000-000000000000'
        ? null
        : raw['Подразделение_Key'],
      comment: raw['Комментарий'],
      sownArea: raw['ПосевнаяПлощадь'],
      rawGreenhouseTopology: raw['ТопологияТеплицы'],
      greenHouseTopology: null,
      rawWorkParameters: raw['НастройкиПараметровВыполненияРабот'],
      workParameters: null,
      raw,
    };

    const result: IResult = {
      payload: {
        value: JSON.stringify(greenhouse),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      entity: {
        value: 'greenhouses',
        type: 'String'
      },
      id: {
        value: greenhouse.id,
        type: 'String'
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};