import { Variables } from 'camunda-external-task-client-js';

import _ from 'lodash';

import logging from '../logging';
import axios from 'axios';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const record = payload.raw;

    const historyId = record['Сотрудник_Key'];

    const backendUrl: string = variables.BACKEND_URL + '/position_histories?id=' + historyId + '&period[strictly_after]=' + record['Period'] + '&page=1&itemsPerPage=1';

    const positionsCount = await axios({
      method: 'get',
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
        'X-Username': 'microservice [POS HISTORY]',
      },
      url: encodeURI(backendUrl),
    })
      .then(response => {
        return response.data.length;
      });

    const positionData = {
      id: positionsCount > 0 ? null : historyId,
      workerId: record['Сотрудник_Key'],
      positionId:
        record['Должность_Key'] == '00000000-0000-0000-0000-000000000000'
          ? null
          : record['Должность_Key'],
      departmentId:
        record['Подразделение_Key'] ==
          '00000000-0000-0000-0000-000000000000'
          ? null
          : record['Подразделение_Key'],
      period: record['Period'],
      active: record['Active'],
      event: record['ВидСобытия'],
      raw: record,
      isProcessed: false
    };

    const result: IResult = {
      payload: {
        value: JSON.stringify(positionData),
        type: 'Object',
        valueInfo: {
          objectTypeName: 'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json',
        },
      },
      entity: {
        value: 'position_histories',
        type: 'String'
      },
      id: {
        value: positionsCount > 0 ? null : positionData.id, // do not import position after already imported
        type: 'String',
      },
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
