import { Variables } from 'camunda-external-task-client-js';

import _ from 'lodash';

import logging from '../logging';
import axios from 'axios';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  collection: object;
  entity: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const collection = [];

    if (!_.isEmpty(raw['RecordSet'])) {
      for (const record of raw['RecordSet']) {

        const historyId = record['Сотрудник_Key'];

        const backendUrl: string = variables.BACKEND_URL + '/position_histories?id=' + historyId + '&period[strictly_after]=' + record['Period'] + '&page=1&itemsPerPage=1';

        const positionsCount = await axios({
          method: 'get',
          headers: {
            Accept: 'application/json',
            'Content-Type': 'application/json',
            'X-Username': 'microservice [RD POS HISTORY]',
          },
          url: encodeURI(backendUrl),
        })
          .then(response => {
            return response.data.length;
          });

        const position_history = {
          // tslint:disable-next-line: no-null-keyword
          id: positionsCount > 0 ? null : historyId,
          workerId: record['Сотрудник_Key'],
          positionId:
            record['Должность_Key'] == '00000000-0000-0000-0000-000000000000'
              // tslint:disable-next-line: no-null-keyword
              ? null
              : record['Должность_Key'],
          departmentId:
            record['Подразделение_Key'] ==
            '00000000-0000-0000-0000-000000000000'
              // tslint:disable-next-line: no-null-keyword
              ? null
              : record['Подразделение_Key'],
          period: record['Period'],
          active: record['Active'],
          event: record['ВидСобытия'],
          raw: record,
          isProcessed: false
        };

        collection.push(position_history);
      }
    }

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   collection,
    // );

    const result: IResult = {
      collection: {
        value: JSON.stringify(collection),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.ArrayList<java.util.LinkedHashMap<java.lang.Object,java.lang.Object>>',
          serializationDataFormat: 'application/json'
        }
      },
      entity: {
        value: 'position_histories',
        type: 'String'
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
