import { Variables } from "camunda-external-task-client-js";

import logging from "../logging";

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const position = {
      id: raw.Ref_Key,
      name: raw["Description"],
      raw: raw
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   position,
    // );

    const result: IResult = {
      payload: {
        value: JSON.stringify(position),
        type: "Object",
        valueInfo: {
          objectTypeName:
            "java.util.HashMap<java.lang.Object,java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      },
      entity: {
        value: "positions",
        type: "String"
      },
      id: {
        value: position.id,
        type: "String"
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
