import { Variables } from "camunda-external-task-client-js";

import _ from "lodash";

import logging from "../logging";

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const type =
      raw["Description"] == "Фото"
        ? "binary"
        : _.includes(raw["ValueType"]["Types"], "Number")
        ? "number"
        : "string";

    const property = {
      id: raw.Ref_Key,
      name: "" + raw["Description"],
      description: "" + raw["Комментарий"],
      regexp:
        type === "number"
          ? "\\\\d{0," +
            (raw["ValueType"]["NumberQualifiers"]["Digits"] === 0
              ? 3
              : raw["ValueType"]["NumberQualifiers"]["Digits"]) +
            "}"
          : null,
      type: type,
      raw: raw
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   property,
    // );

    const result: IResult = {
      payload: {
        value: JSON.stringify(property),
        type: "Object",
        valueInfo: {
          objectTypeName:
            "java.util.HashMap<java.lang.Object,java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      },
      entity: {
        value: "properties",
        type: "String"
      },
      id: {
        value: property.id,
        type: "String"
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
