import { Variables } from 'camunda-external-task-client-js';
import getUuid from 'uuid-by-string';

import logging from '../logging';
import axios from 'axios';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw ;

    const uuidHash = getUuid(raw['Сотрудник_Key'] + raw['ВидРабот_Key']);

    const backendUrl: string = variables.BACKEND_URL + '/ratings?id=' + uuidHash + '&period[strictly_after]=' + raw['Period'] + '&page=1&itemsPerPage=1';

    const ratingsCount = await axios({
      method: 'get',
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
        'X-Username': 'microservice [RD RATING]',
      },
      url: encodeURI(backendUrl),
    })
      .then(response => {
        return response.data.length;
      });

    const ratingData = {
      id: uuidHash, // record['Recorder'],
      period: raw['Period'],
      rating: raw['Рейтинг'],
      ratingInt: raw['РейтингИнтерфейса'],
      raw: raw,
      rawWorkerId: raw['Сотрудник_Key'] === '00000000-0000-0000-0000-000000000000'
        ? null
        : raw['Сотрудник_Key'],
      rawSectionId: raw['ВидРабот_Key'] === '00000000-0000-0000-0000-000000000000'
        ? null
        : raw['ВидРабот_Key'],
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   ratingData,
    // );

    const result: IResult = {
      payload: {
        value: JSON.stringify(ratingData),
        type: 'Object',
        valueInfo: {
          objectTypeName: 'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json',
        },
      },
      entity: {
        value: 'ratings',
        type: 'String',
      },
      id: {
        value: ratingsCount > 0 ? null : ratingData.id, // do not import ratings after already imported
        type: 'String',
      },
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
