import { Variables } from "camunda-external-task-client-js";

import logging from "../logging";

interface IData {
  payload: string;
  type: string;
}
  
interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const rowState = {
      id: raw.Ref_Key,
      description: raw["Description"],
      name: raw["PredefinedDataName"],
      raw,
    }

    const result: IResult = {
      payload: {
        value: JSON.stringify(rowState),
        type: "Object",
        valueInfo: {
          objectTypeName:
            "java.util.HashMap<java.lang.Object,java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      },
      entity: {
        value: "row_states",
        type: "String"
      },
      id: {
        value: rowState.id,
        type: "String"
      }
    }

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch(error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
}