import { Variables } from 'camunda-external-task-client-js';
import getUuid from 'uuid-by-string';

import logging from '../logging';
import axios from 'axios';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  payload: string;
  type: string;
}
  
interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;
    const uuid =  getUuid(raw['Теплица_Key'] + raw['Ряд']);

    const backendUrl: string = variables.BACKEND_URL + '/row_statuses?id=' + uuid + '&period[strictly_after]=' + raw['Period'] + '&page=1&itemsPerPage=1';

    const statusCount = await axios({
      method: 'get',
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
        'X-Username': 'microservice [RD ROW STATS]',
      },
      url: encodeURI(backendUrl),
    })
      .then(response => {
        return response.data.length;
      });

    const rowStatus = {
      id: uuid,
      rowNumber: raw['Ряд'],
      period: raw['Period'],
      greenHouseId: null,
      rawGreenHouseId: '' + raw['Теплица_Key'],
      stateId: null,
      rawStateId: '' + raw['СостояниеРяда_Key'],
      raw,
    };

    const result: IResult = {
      payload: {
        value: JSON.stringify(rowStatus),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      entity: {
        value: 'row_statuses',
        type: 'String'
      },
      id: {
        value: statusCount > 0 ? null : rowStatus.id, // do not import statuses after already imported
        type: 'String'
      }
    }

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch(error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
}