import { Variables } from "camunda-external-task-client-js";

import logging from "../logging";

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const section = {
      id: raw.Ref_Key,
      name: "" + raw["Description"],
      description: "" + (raw["актКомментарий"] || raw["Комментарий"]),
      requireFinish: !(
        raw["актНеТребуетЗавершения"] || raw["НеТребуетЗавершения"]
      ),
      rawParentId:
        raw["Parent_Key"] === "00000000-0000-0000-0000-000000000000"
          ? null
          : raw["Parent_Key"],
      raw: raw
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   section,
    // );

    const result: IResult = {
      payload: {
        value: JSON.stringify(section),
        type: "Object",
        valueInfo: {
          objectTypeName:
            "java.util.HashMap<java.lang.Object,java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      },
      entity: {
        value: "sections",
        type: "String"
      },
      id: {
        value: section.id,
        type: "String"
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
