/* tslint:disable no-null-keyword */

import { Variables } from 'camunda-external-task-client-js';

import logging from '../logging';

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const structure = {
      id: raw.Ref_Key,
      name: raw['Code'],
      description: raw['Description'],
      parentId: null,
      rawParentId: raw['Parent_Key'],
      isFolder: raw['IsFolder'],
      comment: raw['Комментарий'],
      rawGreenhouseTopology: raw['ТопологияТеплицы'],
      greenHouseTopology: null,
      greenHouseId: null,
      rawGreenHouseId: raw['Owner_Key'],
      startedAt: raw['ДатаНачала'],
      finishedAt: raw['ДатаОкончания'],
      regularExpr: raw['РегулярноеВыражение'],
      landingArea: raw['ПлощадьПосадки'],
      batonicalVarietyId: null,
      rawBatonicalVarietyId: raw['БотаническийСорт_Key'],
      raw,
    };

    const result: IResult = {
      payload: {
        value: JSON.stringify(structure),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      entity: {
        value: 'structures',
        type: 'String'
      },
      id: {
        value: structure.id,
        type: 'String'
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};