import { Variables } from 'camunda-external-task-client-js';

import logging from '../logging';
import axios from 'axios';

import moment from 'moment';

interface IConfig {
  BACKEND_URL: string;
}

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

const mapper = {
  'd92adf67-3760-11e7-b77d-005056ac182c': '1abec4c4-0d22-11e6-942a-005056ac0e86', // 1
  'df65b893-3760-11e7-b77d-005056ac182c': '10a6ab25-0d22-11e6-942a-005056ac0e86', // 2
  'de965dd9-c104-11e6-80f8-40a8f0260032': '230e8905-ff37-11e6-80fe-40a8f0260032', // 3
  'de965ddb-c104-11e6-80f8-40a8f0260032': '2b8e7299-ff37-11e6-80fe-40a8f0260032', // 4
  '27bdce5c-9566-11e8-a2d0-005056a398e7': 'ab4581b0-9f8b-11e8-a2d6-005056a3e110', // 5
  '5f519a99-a9ae-11e7-a90f-005056ac182c': '7698def5-d66c-11e7-8110-40a8f0260033', // 6
  '7cf8a7ee-a9ae-11e7-a90f-005056ac182c': '880954d5-d66c-11e7-8110-40a8f0260033', // 7 
  'b95c5736-8397-11e9-a2e9-005056a3e110': '9890c293-da19-11e9-a2f2-005056a3e110', // 8
  'c64299dc-8397-11e9-a2e9-005056a3e110': 'a5ae0818-da19-11e9-a2f2-005056a3e110', // 9
};

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    const taskId = raw.Ref_Key;

    const backendUrl: string = variables.BACKEND_URL + '/tasks?id=' + taskId + '&exists[parentId]=true&page=1&itemsPerPage=1';

    const taskCount = await axios({
      method: 'get',
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json',
        'X-Username': 'microservice [RD TASK]',
      },
      url: encodeURI(backendUrl),
    })
      .then(response => {
        return response.data.length;
      });

    const taskData = {
      id: raw.Ref_Key,
      name: '' + raw['Number'],
      description: '' + raw['Комментарий'],
      requireFinish: false,
      isFinished: raw['Факт'],
      startAt: raw['ДатаНачала'],
      finishAt: raw['ДатаВыполнения'],
      properties: null,
      requireOnlineStart: true,
      orderBy: 0,
      lockedTill: moment().locale('ru').toISOString(),
      //  raw["Исполнитель_Type"] === "StandardODATA.Catalog_Должности" ? 10 : 0,
      // color:
      //  raw["Исполнитель_Type"] === "StandardODATA.Catalog_Должности"
      //    ? "#FFFFFF"
      //    : "#FFC2B3",
      raw: raw,
      rawWorkerId: raw['Исполнитель_Type'] === 'StandardODATA.Catalog_Сотрудники'
        ? raw['Исполнитель']
        : null,
      rawSectionId: raw['ВидРабот_Key'] === '00000000-0000-0000-0000-000000000000'
        ? null
        : raw['ВидРабот_Key'],
      rawDepartmentId: raw['Подразделение_Key'] === '00000000-0000-0000-0000-000000000000'
        ? null
        : raw['Подразделение_Key'],
      rawPositionId: raw['Исполнитель_Type'] === 'StandardODATA.Catalog_Должности'
        ? raw['Исполнитель']
        : null,
      rawProperties: raw['ПараметрыВыполненияРабот'] || null,
      rating_section_id: null,
      row_amount: null,
      row_number: null,
      rating: raw['РейтингМУП'] || 0,
      toSend: false,
      pass: 0,
      toExchange: false,
      greenhouseId: mapper[raw['Подразделение_Key']] || null,
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   taskData,
    // );

    const result: IResult = {
      payload: {
        value: JSON.stringify(taskData),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      entity: {
        value: 'tasks',
        type: 'String'
      },
      id: {
        value: taskCount > 0 ? null : taskData.id, // do not import tasks which have parentId in our system
        type: 'String'
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
