import { Variables } from 'camunda-external-task-client-js';

import _ from 'lodash';
import moment from 'moment';

import logging from '../logging';

interface IData {
  payload: string;
  type: string;
}

interface IResult {
  payload: object;
  entity: object;
  id: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const raw = payload.raw;

    let code = '';

    const additionalReqs = _.find(raw['ДополнительныеРеквизиты'], {
      Свойство_Key: 'e4c03aaf-0ddf-11e6-942a-005056ac0e86'
    });

    if (!!additionalReqs) {
      code = additionalReqs['Значение'];
    }

    const worker = {
      id: raw.Ref_Key,
      name: raw['Description'],
      code: code,
      rating: 0,
      raw: raw,
      isActive: false,
      spreadTill: moment().format(),
      taskCount: 0,
    };

    // console.log(
    //   JSON.stringify(payload, null, 2),
    //   worker,
    // );

    const result: IResult = {
      payload: {
        value: JSON.stringify(worker),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.HashMap<java.lang.Object,java.lang.Object>',
          serializationDataFormat: 'application/json'
        }
      },
      entity: {
        value: 'workers',
        type: 'String'
      },
      id: {
        value: worker.id,
        type: 'String'
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
