import { Variables } from "camunda-external-task-client-js";

import redis from "../redis";
import logging from "../logging";

interface IConfig {
  REDIS_URL: string;
}

interface IData {
  key: string;
  rangeMin: string;
  rangeMax: string;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const redisClient = redis({ url: variables.REDIS_URL });

    const data = await redisClient.zrangebyscore(
      variables.key,
      variables.rangeMin,
      variables.rangeMax
    );

    await taskService.complete(
      task,
      new Variables().setTyped("collection", {
        value: JSON.stringify(data),
        type: "object",
        valueInfo: {
          objectTypeName: "java.util.ArrayList<java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      })
    );
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
