import { Variables } from "camunda-external-task-client-js";

import * as R from "ramda";

import redis from "../redis";
import logging from "../logging";

interface IConfig {
  REDIS_URL: string;
}

interface IData {
  key: string;
  payload: string;
}

interface IResult {
  payload: string;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    const redisClient = redis({ url: variables.REDIS_URL });

    const raw = R.compose(
      R.flatten,
      R.map(data => [Date.now(), JSON.stringify(data)])
    )(R.is(Array, payload) ? payload : [payload]);

    const data = await redisClient.zadd([variables.key, ...raw]);

    const result: IResult = { payload: JSON.stringify(data) };

    await taskService.complete(task, new Variables().setAll(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
