import { Variables } from "camunda-external-task-client-js";

import _ from "lodash";
import uuidv4 from "uuid/v4";

import logging from "../logging";

interface IData {
  payload: string;
  workerId: string;
}

interface IResult {
  payload: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const payload = JSON.parse(variables.payload);

    payload.parentId = payload.id;
    payload.id = uuidv4();
    //payload.startedAt = payload.startedAt;

    payload.raw = [];

    if (payload.sectionId) {
      payload.section = "/sections/" + payload.sectionId;
    }

    if (payload.departmentId) {
      payload.department = "/departments/" + payload.departmentId;
    }

    if (payload.positionId) {
      payload.position = "/positions/" + payload.positionId;
    }

    payload.worker = "/workers/" + variables.workerId;

    delete payload["@context"];
    delete payload["@id"];
    delete payload["@type"];

    const result: IResult = {
      payload: {
        value: JSON.stringify(payload),
        type: "Object",
        valueInfo: {
          objectTypeName:
            "java.util.HashMap<java.lang.Object,java.lang.Object>",
          serializationDataFormat: "application/json"
        }
      }
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
