/* tslint:disable no-null-keyword */

import { Variables } from 'camunda-external-task-client-js';
import _ from 'lodash';
import moment from 'moment';
import getUuid from 'uuid-by-string';
import constants from '../const/constants';

import logging from '../logging';

interface IData {
  payload: string;
}

interface IResult {
  tasks: object;
}

export default async ({ task, taskService }) => {
  const variables: IData = task.variables.getAll();

  try {
    const tasksOutput = [];

    const taskData = JSON.parse(variables.payload);

    const properties = [];

    for (const property of taskData.rawProperties) {
      const mpProperty = _.find(taskData.properties, (o) => {
        const propStage = property['ПриЗавершении'] ? 'finish' : 'start';

        return o.id == property['Параметр_Key']
          && o.stage == propStage;
      });

      if (!_.isEmpty(mpProperty)) {
        const cProperty = _.cloneDeep(property);
        cProperty['РезультатВыполнения'] = mpProperty.id === constants.rowId ? taskData.rowNumber : mpProperty.value;
        cProperty['РегулярноеВыражение'] = mpProperty.regexp;

        properties.push(cProperty);
      }
    }

    const now = moment();

    let startedAt = taskData.startedAt;

    const payload = {
      'Факт': taskData.isFinished,
      'Исполнитель': taskData.workerId ? taskData.workerId : '',
      'Исполнитель_Type': taskData.workerId ? 'StandardODATA.Catalog_Сотрудники' : 'StandardODATA.Undefined',
      'Date': now.format(),
      'ДатаНачала': null, // now.format(),
      'ДатаВыполнения': null, // taskData.finishAt, // now.format(),
    };

    if (!_.isEmpty(properties)) {
      payload['ПараметрыВыполненияРабот'] = properties;
    }

    if (startedAt) {
      payload['ДатаНачала'] = startedAt;
      payload['Date'] = startedAt;
    }

    if (taskData.finishedAt && taskData.isFinished) {
      payload['ДатаВыполнения'] = taskData.finishedAt;
      // payload['Date'] = taskData.finishedAt;
    }

    payload['Ref_Key'] = taskData.id;

    const mergedPayload = _.merge(taskData.raw, payload);

    delete mergedPayload['Number'];

    tasksOutput.push({
      post: {...mergedPayload},
      patch: {...payload},
    });

    for (const property of taskData.properties) {
      if (!property.parentId || property.parentId === property.id || _.isEmpty(property.value)) {
        continue;
      }

      const mProperties = _.cloneDeep(properties);
      const mPayload = _.cloneDeep(payload);

      _.each(mProperties, (o) => {
        if (o['Параметр_Key'] === property.parentId) {
          o['РезультатВыполнения'] = property.value;
        }
      });

      mPayload.Ref_Key = getUuid(taskData.id + '-' + property.id);
      mPayload['ПараметрыВыполненияРабот'] = mProperties;

      if (property.timestamp) {
        startedAt = property.timestamp;
      }

      if (startedAt) {
        mPayload['ДатаНачала'] = startedAt;
        mPayload['Date'] = startedAt;
      }

      const mMergedPayload = _.merge(_.cloneDeep(mergedPayload), mPayload);

      const previousPair = _.last(tasksOutput);

      if (startedAt) {
        previousPair.post['ДатаВыполнения'] = startedAt;
        previousPair.patch['ДатаВыполнения'] = startedAt;
      }

      previousPair.post['Факт'] = true;
      previousPair.patch['Факт'] = true;

      tasksOutput.push({
        post: mMergedPayload,
        patch: mPayload,
      });
    }

    // console.log(JSON.stringify(tasksOutput, null, 4));

    const result: IResult = {
      tasks: {
        value: JSON.stringify(tasksOutput),
        type: 'Object',
        valueInfo: {
          objectTypeName:
            'java.util.ArrayList<java.util.LinkedHashMap<java.lang.Object,java.lang.Object>>',
          serializationDataFormat: 'application/json'
        }
      },
    };

    await taskService.complete(task, new Variables().setAllTyped(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
