import { Variables } from 'camunda-external-task-client-js';

import axios from 'axios';

import logging from '../logging';

interface IConfig {
  oneCUrl: string;
}

interface IUserData {
  userId: number;
}

interface IResult {
  isSuccessful: boolean;
}

export default async ({ task, taskService }) => {
  const variables: IConfig & IUserData = task.variables.getAll();

  try {
    await axios({
      method: 'get',
      url: variables.oneCUrl + 'workers/' + variables.userId,
      headers: {
        'X-Username': 'microservice [USER AUTH]',
      }
    });

    const result: IResult = { isSuccessful: true };

    await taskService.complete(task, new Variables().setAll(result));
  } catch (error) {
    await logging({ error, variables, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.response.statusText
    });
  }
};
