import { Variables } from "camunda-external-task-client-js";
import getUuid from "uuid-by-string";
import _ from "lodash";

import uuidv4 from "uuid/v4";

import logging from "../logging";

interface IData {
  seed1: string;
  seed2: string;
}

interface IResult {
  id: string;
}

export default async ({ task, taskService }) => {
  try {

    const variables: IData = task.variables.getAll();

    const uuidHash = (_.isNull(variables.seed1) ? "" : variables.seed1) + (_.isNull(variables.seed2) ? "" : variables.seed2);

    const result: IResult = {
      id: uuidv4()
    };

    if(!_.isEmpty(uuidHash)) {

      result.id = getUuid(uuidHash);

    }

    await taskService.complete(task, new Variables().setAll(result));
  } catch (error) {
    await logging({ error, variables: {}, filename: __filename });
    await taskService.handleFailure(task, {
      errorMessage: error.message
    });
  }
};
